package org.acme;

import io.quarkus.test.junit.QuarkusTest;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.WebApplicationException;
import org.acme.log.LogHandler;
import org.apache.http.HttpStatus;
import org.eclipse.microprofile.rest.client.inject.RegisterRestClient;
import org.eclipse.microprofile.rest.client.inject.RestClient;
import org.jboss.logmanager.Logger;
import org.jboss.resteasy.annotations.GZIP;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;

@QuarkusTest
class GzipTest {

    private static final LogHandler logHandler = new LogHandler();
    @RestClient
    MyClient client;

    @BeforeAll
    static void beforeAll() {
        Logger.getLogger("org.jboss.resteasy.resteasy_jaxrs.i18n").addHandler(logHandler);
    }
    @AfterAll
    static void afterAll() {
        Logger.getLogger("org.jboss.resteasy.resteasy_jaxrs.i18n").removeHandler(logHandler);
    }

    /**
     * Test that covers the property `quarkus.resteasy.gzip.max-input`.
     * Larger payloads than 10 megabytes should return HTTP Request Too Large.
     */
    @Test
    void testGzipMaxInput() {
        WebApplicationException ex = Assertions.assertThrows(WebApplicationException.class, () -> client.gzip(new byte[20000000]));
        assertEquals(HttpStatus.SC_REQUEST_TOO_LONG, ex.getResponse().getStatus());
        assertEquals("RESTEASY002137: Invalid format for resteasy.gzip.max.input, using default value 10000000", logHandler.getLoggedMessagesFormatted().get(0));
        assertEquals("Worked!", client.gzip(new byte[10000000]));
    }

    @RegisterRestClient(configKey = "my-client")
    public interface MyClient {

        @POST
        @Path("/gzip")
        String gzip(@GZIP byte[] message);

    }
}
